@@ -18,6 +18,35 @@ function log_message($message) {
     error_log("[$timestamp] BackupWorker: $message");
 }
 
+function cleanup_stuck_backups() {
+    try {
+        $mysql = new mysqlmgmt();
+        $db = $mysql->getMySQLConnection();
+        $db->exec("USE whp");
+
+        // Find backups stuck in 'running' or 'claimed' state for more than 30 minutes
+        $stmt = $db->prepare("
+            UPDATE backup_history
+            SET status = 'failed',
+                error_message = 'Backup timed out or worker died',
+                completed_at = NOW()
+            WHERE status IN ('running', 'claimed')
+            AND started_at < DATE_SUB(NOW(), INTERVAL 30 MINUTE)
+        ");
+        $stmt->execute();
+        $affectedRows = $stmt->rowCount();
+
+        if ($affectedRows > 0) {
+            log_message("Cleaned up $affectedRows stuck backup(s)");
+        }
+
+        return $affectedRows;
+    } catch (Exception $e) {
+        log_message("Error cleaning up stuck backups: " . $e->getMessage());
+        return 0;
+    }
+}
+
 function process_pending_backups() {
     try {
         $mysql = new mysqlmgmt();
@@ -105,6 +134,15 @@ function process_pending_backups() {
 if (php_sapi_name() === 'cli') {
     $lockFile = '/tmp/backup-worker.lock';
 
+    // Check if lock file exists and is stale (older than 1 hour)
+    if (file_exists($lockFile)) {
+        $lockAge = time() - filemtime($lockFile);
+        if ($lockAge > 3600) {
+            log_message("Found stale lock file (age: {$lockAge} seconds), removing it");
+            unlink($lockFile);
+        }
+    }
+
     // Create lock file with current PID
     $pid = getmypid();
     if (file_put_contents($lockFile, $pid) === false) {
@@ -113,15 +151,47 @@ if (php_sapi_name() === 'cli') {
     }
 
     // Register shutdown function to clean up lock file
-    register_shutdown_function(function() use ($lockFile) {
+    register_shutdown_function(function() use ($lockFile, $pid) {
+        // Only remove lock if it still contains our PID
         if (file_exists($lockFile)) {
-            unlink($lockFile);
+            $currentPid = trim(file_get_contents($lockFile));
+            if ($currentPid == $pid) {
+                unlink($lockFile);
+                log_message("Lock file cleaned up by shutdown handler");
+            }
         }
     });
 
+    // Set up signal handlers for clean shutdown
+    if (function_exists('pcntl_signal')) {
+        pcntl_signal(SIGTERM, function() use ($lockFile, $pid) {
+            log_message("Received SIGTERM, cleaning up...");
+            if (file_exists($lockFile)) {
+                $currentPid = trim(file_get_contents($lockFile));
+                if ($currentPid == $pid) {
+                    unlink($lockFile);
+                }
+            }
+            exit(0);
+        });
+        pcntl_signal(SIGINT, function() use ($lockFile, $pid) {
+            log_message("Received SIGINT, cleaning up...");
+            if (file_exists($lockFile)) {
+                $currentPid = trim(file_get_contents($lockFile));
+                if ($currentPid == $pid) {
+                    unlink($lockFile);
+                }
+            }
+            exit(0);
+        });
+    }
+
     log_message("Backup worker started (PID: $pid)");
 
     try {
+        // First, clean up any stuck backups
+        cleanup_stuck_backups();
+
         // Process pending backups
         process_pending_backups();
     } catch (Exception $e) {
@@ -132,7 +202,10 @@ if (php_sapi_name() === 'cli') {
 
     // Clean up lock file
     if (file_exists($lockFile)) {
-        unlink($lockFile);
+        $currentPid = trim(file_get_contents($lockFile));
+        if ($currentPid == $pid) {
+            unlink($lockFile);
+        }
     }
 } else {
     echo "This script can only be run from command line\n";
